package fmon.builder

import java.io.File
import java.util.prefs.Preferences

import scala.util.Properties

import javafx.fxml.FXML
import javafx.fxml.FXMLLoader
import javafx.fxml.JavaFXBuilderFactory
import javafx.scene.{control => jfxsc, layout => jfxsl}

import scalafx.Includes._
import scalafx.stage.FileChooser
import FileChooser.ExtensionFilter

trait Savable {
  def saveTo(file: File): Unit
  def openFrom(file: File): Unit
}

class App {
  @FXML var pane: jfxsl.Pane = _
  
  var builder: Savable = _
  var lastFile: File = _
  
  def lastDir(): String = {
    val prefs = Preferences.userNodeForPackage(classOf[App])
    prefs.get("dir", Properties.userDir)
  }
  
  def rememberDir(file: File): Unit = {
    val dir = if (file.isDirectory()) file.getPath else file.getParent
    if (dir != lastDir) {
      val prefs = Preferences.userNodeForPackage(classOf[App])
      prefs.put("dir", dir)
    }
  }
  
  def open(): Unit = {
    val fileChooser = new FileChooser {
      title = "Open Resource File"
      initialDirectory = new File(lastDir())
      extensionFilters ++= Seq(
        new ExtensionFilter("YAML Files", "*.yaml"),
        new ExtensionFilter("All Files", "*.*")
      )
    }
    
    val selectedFile = fileChooser.showOpenDialog(null)
    if (selectedFile != null) {
      rememberDir(selectedFile)
      builder openFrom selectedFile
      lastFile = selectedFile
    }
  }
  
  def save(): Unit = {
    if (lastFile != null) {
      builder saveTo lastFile
    } else {
      saveAs()
    }
  }
  
  def saveAs(): Unit = {
    val fileChooser = new FileChooser {
      title = "Open Resource File"
      initialDirectory = new File(lastDir())
      extensionFilters ++= Seq(
        new ExtensionFilter("YAML Files", "*.yaml"),
        new ExtensionFilter("All Files", "*.*")
      )
    }
    
    val selectedFile = fileChooser.showSaveDialog(null)
    if (selectedFile != null) {
      rememberDir(selectedFile)
      builder saveTo selectedFile
      lastFile = selectedFile
    }
  }
}