package fmon.draw

import java.io.{File, FileInputStream}

import javafx.animation.{Transition => JTransition}

import scalafx.Includes._
import scalafx.animation._
import scalafx.scene.image.ImageView
import scalafx.util.Duration

import fmon.util.YamlHelper

class SpriteAnimation(val view: ImageView, val image: Drawable, val duration: Duration) extends JTransition {
  setCycleDuration(duration)
  setInterpolator(Interpolator.Linear)
  setAutoReverse(false)
  
  def interpolate(t: Double): Unit = {
    image.draw(view, t)
  }
}

class AnimatedImage(val frames: IndexedSeq[Drawable]) extends Drawable {
  val lastIndex = -1
  
  def draw(view: ImageView, t: Double): Unit = {
    val index = Math.min((frames.size * t).toInt, frames.size - 1)
    frames(index).draw(view, t)
  }
}

case class AnimationToken(val filename: String, val indices: IndexedSeq[Int]) {
  def load(dir: File) = {
    val file = new File(s"${dir.getPath}/${filename}")
    val palette = Palette.square(new FileInputStream(file), 5)
    val images = palette.images
    val frames = indices.map(i => images(i))
    new AnimatedImage(frames)
  }
}

object AnimationToken {
  val tokens = YamlHelper.extractMap[AnimationToken](new FileInputStream(raw"C:\Users\dalyj\Documents\Design\animations.yaml"))
  
  def apply(name: String) = tokens(name)
}