package fmon.draw

import java.io._

object Battler {
  final val FramesPerPose = 3
  
  def apply(filename: String): Sprite = {
    this(new File(filename))
  }
  
  def apply(file: File): Sprite = {
    val palette = new Palette(new FileInputStream(file), 9, 6)
    def pose(x: Int, y: Int): AnimatedImage = {
      val xx = x * FramesPerPose
      val frames = IndexedSeq(palette(xx, y), palette(xx + 1, y), palette(xx + 2, y), palette(xx + 1, y))
      new AnimatedImage(frames)
    }
    val poses = Map(
        "Idle" -> pose(0, 0),
        "ReadyPhys" -> pose(0, 1),
        "ReadyMag" -> pose(0, 2),
        "Guard" -> pose(0, 3),
        "Damage" -> pose(0, 4),
        "Evade" -> pose(0, 5),
        "Stab" -> pose(1, 0),
        "Swing" -> pose(1, 1),
        "Shoot" -> pose(1, 2),
        "PhysSkill" -> pose(1, 3),
        "MagSkill" -> pose(1, 4),
        "Item" -> pose(1, 5),
        "Escape" -> pose(2, 0),
        "Victory" -> pose(2, 1),
        "Crisis" -> pose(2, 2),
        "Abnormal" -> pose(2, 3),
        "Sleep" -> pose(2, 4),
        "Dead" -> pose(2, 5)
    )
    new Sprite("Idle", poses)
  }
}

import scalafx.application.JFXApp
import scalafx.scene._
import scalafx.scene.control._
import scalafx.scene.image._
import scalafx.scene.paint.Color._
import scalafx.util.Duration

object DrawingDemo extends JFXApp {
  val view = new ImageView
  stage = new JFXApp.PrimaryStage {
    title.value = "Hello Stage"
    width = 600
    height = 450
    scene = new Scene {
      fill = LightGreen
      content = view
    }
  }
  val battler = Battler(raw"C:\Users\James\Documents\Design\fakemon\images\Aekashics Librarium MV Format Animated Battlers\Aekashics Librarium MV Format Animated Battlers\RPG Maker MV format Animated Sideview Battlers\Bat\Bat1_1.png")
  val animation = new SpriteAnimation(view, battler, new Duration(1000)) {
    setCycleCount(10)
  }
  animation.play()
  
}