package fmon.stat

import scala.reflect.runtime.universe._
import scala.tools.reflect.ToolBox
import scala.io.Source
import scala.util.Random

import fmon.battle.msg.SignalConsumer
import fmon.util._

abstract class Ability extends Effect {
  val name: String
  val desc: String
  
  def effectType = EffectType.AbilityEffect
  
  def onAfterDamage(source: Monster, move: MoveTurn, mon: Monster, dmg: Int)(implicit reader: SignalConsumer, rng: Random) = 
    onAfterDamageImp(this, source, move, mon, dmg, reader, rng)
  
  protected val onAfterDamageImp: (Ability, Monster, MoveTurn, Monster, Int, SignalConsumer, Random) => Unit
  
  override def toString = name
}

class AbilityToken(
  val name: String,
  val id: String,
  val shortDesc: String,
  val onAfterDamage: String
  ) {
  
  def instantiate(): Ability = {
    val self = this
    new Ability {
      val name = self.name
      val id = self.id
      val desc = self.shortDesc
      val onAfterDamageImp = Ability.compileOnAfterDamage(self.onAfterDamage)
    }
  }
}

object Ability {
  private var abilities = Map[String, Ability]()
  val tokens = YamlHelper.extractMap[AbilityToken](Source.fromInputStream(Ability.getClass.getResourceAsStream("data/abilities.yaml")))
  private val tb = runtimeMirror(getClass.getClassLoader).mkToolBox()
  
  def apply(name: String) = {
    if (!abilities.contains(name)) {
      abilities = abilities.updated(name, tokens(name).instantiate())
    }
    abilities(name)
  }
  
  private val header = """
    |import scala.util.Random
    |import fmon.battle.msg._
    |import fmon._
    |import fmon.stat._
    |import fmon.stat.MoveType._
    |import fmon.stat.Statistic._
    """
  private val helper = """
    implicit val gen = rng
    implicit val signalConsumer = reader
    def msg(text: String): Unit = {
      reader ! Message(text)
    }
    def trigger {
      msg(s"[${mon}'s ${self}]")
    }
    def effect = EffectSource(mon, self)
    """
  
  def compileOnAfterDamage(code: String): (Ability, Monster, MoveTurn, Monster, Int, SignalConsumer, Random) => Unit = {
    if (code == null) {
      (_, _, _, _, _, _, _) => ()
    } else {
      val tree = tb.parse(
        s"""
           |$header
           |def onAfterDamage(self: Ability, source: Monster, move: MoveTurn, mon: Monster, damage: Int, reader: SignalConsumer, rng: Random): Unit = {
           |  $helper
           |  $code
           |}
           |onAfterDamage _
          """.stripMargin)
      val f = tb.compile(tree)
      val wrapper = f()
      implicit val c = code
      wrapper.asInstanceOf[(Ability, Monster, MoveTurn, Monster, Int, SignalConsumer, Random) => Unit]
    }
  }
}