package fmon.stat

import scala.io.Source

import com.fasterxml.jackson.module.scala.JsonScalaEnumeration

import Statistic._

import fmon.util.YamlHelper

abstract class Form {
  val name : String
  //val height
  //val weight
  val desc : String
  val elements : IndexedSeq[Element]
  val baseStats : Map[Stat, Int]
  // val appearance // animation
  // moves
  val abilities : IndexedSeq[Ability]
  val xpCurve : XpCurve
  val catchRate : Int
  // val color
  
  override def toString = {
    name
  }
}

case class FormToken(
    val name : String,
    val desc : String,
    val abilities : IndexedSeq[String],
    val elements : IndexedSeq[String],
    val baseStats : Map[String, Int],
    @JsonScalaEnumeration(classOf[XpCurveType]) val xpCurve : XpCurve,
    val catchRate : Int = 255
    ) {
  
  def instantiate() = {
    val self = this
    new Form {
      val name = self.name
      val desc = self.desc
      val elements = self.elements.map(Element(_))
      val baseStats = self.baseStats.map{ case (s, i) => (Statistic(s), i)}
      val abilities = self.abilities.map(Ability(_))
      val xpCurve = self.xpCurve
      val catchRate = self.catchRate
    }
  }
}

object Form {
  def apply(name : String) = byName(name)
  
  val forms = YamlHelper.extractSeq[FormToken](Form.getClass.getResourceAsStream("data/forms.yaml")).map(_.instantiate())
  val byName = forms.map(f => (f.name, f)).toMap
}