package fmon

import java.io._

import scala.language.implicitConversions

package object util {
  type TypeReference[A] = com.fasterxml.jackson.core.`type`.TypeReference[A]
  
  def using[T <: Closeable, A](resource : T)(block : T => A) : A = {
    try {
      block(resource)
    } finally {
      resource.close()
    }
  }
  
  def makeParentDirs(filename : String) : Unit = {
    val file = new File(filename)
    file.getParentFile().mkdirs()
  }
  
  def getFilename(file : String) : String = {
    file.split("\\.").head
  }
  
  implicit def fileName2Writer(filename : String) : PrintWriter = {
    makeParentDirs(filename)
    new PrintWriter(new BufferedWriter(new FileWriter(filename)))
  }

  implicit def fileName2Reader(filename : String) : BufferedReader = {
    new BufferedReader(new FileReader(filename))
  }
  
  implicit def filename2OutputStream(filename : String) : PrintStream = {
    makeParentDirs(filename)
    new PrintStream(new BufferedOutputStream(new FileOutputStream(filename)))
  }

  implicit def filename2InputStream(filename : String) : BufferedInputStream = {
    new BufferedInputStream(new FileInputStream(filename))
  }
  
}