package fmon.world

import java.io._

import fmon.Config
import fmon.draw.tile._

class Tileset(val groundTiles: IndexedSeq[AutoTile], val doodadTiles: IndexedSeq[AutoTile], val info: IndexedSeq[TileInfo]) {
  def groundInfo(index: Int) = info(index)
  def doodadInfo(index: Int) = info(index + groundTiles.size)
  
  val infoOf = (groundTiles.zipWithIndex.map{case (t, i) => (t, groundInfo(i))} ++ doodadTiles.zipWithIndex.map{case (t, i) => (t, doodadInfo(i))}).toMap
}

case class TileInfo(val flags: Int = TileInfo.PassAll) {
  def doesPassNorth = (flags & TileInfo.PassNorth) != 0
  def doesPassEast = (flags & TileInfo.PassEast) != 0
  def doesPassSouth = (flags & TileInfo.PassSouth) != 0
  def doesPassWest = (flags & TileInfo.PassWest) != 0
  
  def doesOverwrite = (flags & TileInfo.Overwrite) != 0
  
  def isCounter = (flags & TileInfo.IsCounter) != 0
}


object TileInfo {
  val PassNorth = 0x0001
  val PassEast = 0x0002
  val PassSouth = 0x0004
  val PassWest = 0x0008
  val PassAll = PassNorth | PassEast | PassSouth | PassWest
  val NoPass = 0
  
  val Overwrite = 0x0010
  val Underfoot = 0x0020
  val Overhead = 0x0040
  
  val IsCounter = 0x0100
}

case class TilesetToken(
    val name: String,
    val a1: String,
    val a2: String,
    val a3: String,
    val a4: String,
    val a5: String,
    val b: String,
    val c: String,
    val info: IndexedSeq[TileInfo]) {
  
  def load(dir: File): Tileset = {
    val a1Tiles = AutoTilePalette.a2(new File(dir, a1), Config.tileSize).tiles
    val a2Tiles = AutoTilePalette.a2(new File(dir, a2), Config.tileSize).tiles
    val a3Tiles = AutoTilePalette.a3(new File(dir, a3), Config.tileSize).tiles
    val a4Tiles = AutoTilePalette.a4(new File(dir, a4), Config.tileSize).tiles
    val a5Tiles = AutoTilePalette.basic(new File(dir, a5), Config.tileSize).tiles
    val groundTiles = a1Tiles ++ a2Tiles ++ a3Tiles ++ a4Tiles ++ a5Tiles
    val bTiles = AutoTilePalette.partitioned(new File(dir, b), 2, Config.tileSize).tiles
    val cTiles = AutoTilePalette.partitioned(new File(dir, c), 2, Config.tileSize).tiles
    val doodadTiles = bTiles ++ cTiles
    new Tileset(groundTiles, doodadTiles, info)
  }
  
  override def toString = name
}